@@ -1,7 +1,7 @@
-;;; brltty.el --- Interface to BRLTTY
+;;; brltty.el --- Interface to BRLTTY  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2004, 2005, 2006, 2007, 2008 Brailcom, o.p.s.
-;; Copyright (C) 2012, 2013, 2020 Milan Zamazal <pdm@zamazal.org>
+;; Copyright (C) 2012-2021 Milan Zamazal <pdm@zamazal.org>
+;; Copyright (C) 2004-2008 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -25,7 +25,7 @@
 ;;; Code:
 
 
-(require 'cl)
+(require 'cl-lib)
 
 (require 'speechd-common)
 
@@ -36,7 +36,7 @@
 (defgroup brltty ()
   "BRLTTY interface."
   :group 'speechd-el)
-  
+
 (defcustom brltty-default-host "localhost"
   "Default BRLTTY host to connect to."
   :type 'string
@@ -131,7 +131,7 @@ available, from the  environment variable CONTROLVT."
     (?K . key)
     (?C . credentials)))
 
-(defstruct brltty--connection
+(cl-defstruct brltty--connection
   process
   (protocol-version nil)
   (display-width nil)
@@ -151,8 +151,8 @@ available, from the  environment variable CONTROLVT."
 (unless brltty--emacs-process-ok
   (defvar brltty--process-connections '()))
 
-(lexical-let ((last-selected-frame 'uninitialized)
-              (last-terminal-spec 'uninitialized))
+(let ((last-selected-frame 'uninitialized)
+      (last-terminal-spec 'uninitialized))
   (defun brltty--terminal-spec ()
     (if (eq (selected-frame) last-selected-frame)
         last-terminal-spec
@@ -287,8 +287,8 @@ available, from the  environment variable CONTROLVT."
      (brltty--disable-connection connection err))))
 
 (defun brltty--read-input (connection)
-  (destructuring-bind (type . data) (brltty--read-packet connection)
-    (case type
+  (cl-destructuring-bind (type . data) (brltty--read-packet connection)
+    (cl-case type
      (err
       (let ((err-number (brltty--read-integer data)))
         (if (= err-number brltty--protocol-version-error)
@@ -352,14 +352,14 @@ available, from the  environment variable CONTROLVT."
     (cdr answer)))
 
 (defun brltty--send-packet (connection answer packet-id &rest data-list)
-  (let ((length (reduce #'+ data-list :initial-value 0
-                        :key #'(lambda (data)
-                                 (cond 
-				  ((integerp data) 4)
-                                  ((consp data)
-                                   (ecase (car data)
-                                     (integer64 8)))
-				  (t (length data))))))
+  (let ((length (cl-reduce #'+ data-list :initial-value 0
+                           :key #'(lambda (data)
+                                    (cond
+				     ((integerp data) 4)
+                                     ((consp data)
+                                      (cl-ecase (car data)
+                                        (integer64 8)))
+				     (t (length data))))))
         (process (brltty--connection-process connection)))
     (when process
       (with-speechd-coding-protection
@@ -370,10 +370,10 @@ available, from the  environment variable CONTROLVT."
                       (encode-coding-string
                        (apply #'format "%c%c%c%c"
                               (funcall #'reverse
-                                       (loop for i from 1 to 4
-                                             for x = n then (/ x 256)
-                                             for rem = (% x 256)
-                                             collect rem)))
+                                       (cl-loop for i from 1 to 4
+                                                for x = n then (/ x 256)
+                                                for rem = (% x 256)
+                                                collect rem)))
                        brltty-coding))))
               (send-integer length)
               (send-integer (cdr (assoc packet-id brltty--packet-types)))
@@ -387,8 +387,8 @@ available, from the  environment variable CONTROLVT."
                      process
                      (encode-coding-string (format "%c" (aref data i)) brltty-coding))))
                  ((consp data)
-                  (destructuring-bind (n1 n2 n3) (cdr data)
-                    (ecase (car data)
+                  (cl-destructuring-bind (n1 n2 n3) (cdr data)
+                    (cl-ecase (car data)
                       (integer64
                        (process-send-string
                         process
@@ -412,7 +412,7 @@ available, from the  environment variable CONTROLVT."
 (defun brltty--update-terminal-spec (connection)
   (let ((terminal-spec (brltty--terminal-spec))
         (orig-terminal-spec (brltty--connection-terminal-spec connection)))
-    (unless (equalp terminal-spec orig-terminal-spec)
+    (unless (cl-equalp terminal-spec orig-terminal-spec)
       (when orig-terminal-spec
         (brltty--send-packet connection 'ack 'leavetty))
       (apply 'brltty--send-packet
@@ -437,7 +437,7 @@ respectively."
       ;; In protocol >= 8 server initiates communication, let's look if
       ;; there is any
       (let* ((connection (brltty--open-connection host port key-handler))
-             (version (or (first (brltty--read-answer connection 'version t))
+             (version (or (cl-first (brltty--read-answer connection 'version t))
                           7)))
         (if (> version 7)
             (progn
