@@ -1,6 +1,7 @@
-;;; speechd-brltty.el --- BRLTTY output driver
+;;; speechd-brltty.el --- BRLTTY output driver  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2004, 2005, 2006, 2008 Brailcom, o.p.s.
+;; Copyright (C) 2021 Milan Zamazal <pdm@zamazal.org>
+;; Copyright (C) 2004-2008 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -24,8 +25,7 @@
 ;;; Code:
 
 
-(eval-when-compile
-  (require 'cl))
+(require 'cl-lib)
 
 (require 'brltty)
 (require 'mmanager)
@@ -93,8 +93,8 @@ is not recommended to assign or call user commands here."
       (let ((first-time (eq connection 'uninitialized)))
         (if dont-open
             (setq connection nil)
-          (lexical-let ((driver driver))
-            (setq connection (condition-case err 
+          (let ((driver driver))
+            (setq connection (condition-case err
                                  (brltty-open
                                   nil nil
                                   (lambda (key)
@@ -109,7 +109,7 @@ is not recommended to assign or call user commands here."
     connection))
 
 (defun speechd-brltty--display (manager message &optional scroll)
-  (multiple-value-bind (connection text cursor) message
+  (cl-multiple-value-bind (connection text cursor) message
     (let ((display-width (car (brltty-display-size connection))))
       (when display-width
         (when (and cursor (>= cursor display-width) (not scroll))
@@ -153,8 +153,10 @@ is not recommended to assign or call user commands here."
       (mmanager-put manager 'scrolling
                     (if bolp
                         0
-                      (max (- scrolling (or (car (brltty-display-size connection)) 0))
-                           0)))
+                      (let ((display-size (car (brltty-display-size
+                                                (speechd-brltty--connection driver)))))
+                        (max (- scrolling (or display-size 0))
+                             0))))
       (speechd-brltty--display manager (mmanager-history manager 'current) t))))
 
 (defun speechd-brltty-scroll-to-bol (driver)
@@ -168,7 +170,7 @@ is not recommended to assign or call user commands here."
          (message (mmanager-history manager 'current)))
     (when scrolling
       (speechd-braille--stop manager)
-      (destructuring-bind (connection text cursor) message
+      (cl-destructuring-bind (connection text _cursor) message
         (let ((display-width (or (car (brltty-display-size connection)) 0)))
           (when display-width
             (setq scrolling (if eolp
@@ -212,7 +214,7 @@ is not recommended to assign or call user commands here."
   "Discard all messages from the display queue."
   (speechd.cancel driver 'all))
 
-(defun speechd-brltty-command-key (driver key)
+(defun speechd-brltty-command-key (_driver key)
   "Put given key to the command queue."
   (setq unread-command-events (append unread-command-events (list key))))
 
@@ -242,7 +244,7 @@ is not recommended to assign or call user commands here."
    (brltty-last-try-time :initform 0)))
 
 (cl-defmethod initialize-instance :after
-    ((this speechd-brltty-driver) slots)
+    ((this speechd-brltty-driver) _slots)
   (oset this manager (speechd-brltty--create-manager)))
 
 (cl-defmethod speechd-braille--make-message
