@@ -1,7 +1,7 @@
-;;; speechd-bug.el --- reporting speechd-el and speechd bugs
+;;; speechd-bug.el --- reporting speechd-el and speechd bugs  -*- lexical-binding: t -*-
 
+;; Copyright (C) 2012, 2021 Milan Zamazal <pdm@zamazal.org>
 ;; Copyright (C) 2003, 2004, 2005 Brailcom, o.p.s.
-;; Copyright (C) 2012 Milan Zamazal <pdm@zamazal.org>
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -23,9 +23,10 @@
 ;;; Code:
 
 
-(eval-when-compile (require 'cl))
+(require 'cl-lib)
+(require 'reporter)
 (require 'speechd-speak)
-  
+
 
 (defvar speechd-bug--log-extractor "speechd-log-extractor")
 
@@ -35,7 +36,7 @@
   '(("festival-czech") ("festival-freebsoft-utils") ("sbts")
     ("speech-dispatcher") ("speechd-el") ("unknown")))
 
-  
+
 (defvar speechd-bug--repro-id nil)
 
 (defvar speechd-bug--marker nil)
@@ -72,7 +73,7 @@
           (setq directories (cdr directories)))))
     found))
 
-  
+
 ;;; General information insertion
 
 
@@ -81,19 +82,19 @@
   (speechd-bug--insert (format "Version of `%s':" program))
   (shell-command (format "%s --version | head -1" program) t))
 
-(defun speechd-bug--insert-config-file (file directories comment-start)
+(defun speechd-bug--insert-config-file (file directories comment-prefix)
   (speechd-bug--ensure-empty-line)
   (speechd-bug--insert "===" file ":begin===")
   (let ((file-name
          (or (speechd-bug--look-for-file file directories)
-             (condition-case c
+             (condition-case _c
                  (read-file-name
                   (format "Configuration file `%s' not found, please type its location manually: " file))
                (quit)))))
     (if file-name
         (let ((point (point)))
           (insert-file-contents file-name)
-          (flush-lines (format "^[ \t]*\\(%s.*\\)?$" comment-start)
+          (flush-lines (format "^[ \t]*\\(%s.*\\)?$" comment-prefix)
                        point (point-max-marker)))
       (speechd-bug--insert "---not-found---")))
   (speechd-bug--insert "===" file ":end==="))
@@ -176,7 +177,7 @@
           (unwind-protect
               (progn
                 (set-process-filter process
-                                    #'(lambda (p str)
+                                    #'(lambda (_p str)
                                         (setq output (concat output str))))
                 (process-send-string process "server_log_file\n")
                 (while output
@@ -199,8 +200,8 @@
 
 (defun speechd-bug--generate-repro-id ()
   (let ((time (current-time)))
-    (format "speechd-el-%d-%d-%d" (first time) (second time) (third time))))
-  
+    (format "speechd-el-%d-%d-%d" (cl-first time) (cl-second time) (cl-third time))))
+
 (defun speechd-bug-reproduce ()
   "Start reproducing a speechd-el or Speech Dispatcher bug.
 All user and speechd actions are watched from this moment.
@@ -245,7 +246,6 @@ generating new bug report."
 (defun speechd-bug ()
   "Send a bug report on speechd-el or Speech Dispatcher."
   (interactive)
-  (require 'reporter)
   (let ((package (completing-read "Package: " speechd-bug-packages))
         (reporter-prompt-for-summary-p t))
     (reporter-submit-bug-report
