@@ -1,5 +1,6 @@
-;;; speechd-out.el --- Alternative output interface
+;;; speechd-out.el --- Alternative output interface  -*- lexical-binding: t -*-
 
+;; Copyright (C) 2018-2021 Milan Zamazal <pdm@zamazal.org>
 ;; Copyright (C) 2004, 2005, 2006, 2008 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
@@ -22,7 +23,7 @@
 ;;; Code:
 
 
-(eval-when-compile (require 'cl))
+(require 'cl-lib)
 (require 'eieio)
 
 (require 'speechd-common)
@@ -63,8 +64,8 @@
               (push ,$err ,$speechd-out-errors)))))
        (when ,$speechd-out-errors
          ;; How to signal all the errors?
-         (destructuring-bind
-             (,$error . ,$error-data) (first ,$speechd-out-errors)
+         (cl-destructuring-bind
+             (,$error . ,$error-data) (cl-first ,$speechd-out-errors)
            (setq ,$speechd-out-errors (cdr ,$speechd-out-errors))
            (while (and ,$speechd-out-errors
                        (eq (caar ,$speechd-out-errors) ,$error))
@@ -91,7 +92,7 @@
          :reader speechd-driver.name)))
 
 (cl-defmethod initialize-instance :after
-    ((this speechd-driver) slots)
+    ((this speechd-driver) _slots)
   (if (not (slot-boundp this 'name))
     (error "Driver name not given")))
 
@@ -107,43 +108,43 @@
                  (cons (car list) (replace (cdr list)))))))
       (setq speechd-out--drivers (replace speechd-out--drivers)))))
 
-(cl-defmethod speechd.cancel ((driver speechd-driver) all))
+(cl-defmethod speechd.cancel ((_driver speechd-driver) _all))
 
-(cl-defmethod speechd.stop ((driver speechd-driver) all))
+(cl-defmethod speechd.stop ((_driver speechd-driver) _all))
 
-(cl-defmethod speechd.pause ((driver speechd-driver) all))
+(cl-defmethod speechd.pause ((_driver speechd-driver) _all))
 
-(cl-defmethod speechd.resume ((driver speechd-driver) all))
+(cl-defmethod speechd.resume ((_driver speechd-driver) _all))
 
-(cl-defmethod speechd.repeat ((driver speechd-driver)))
+(cl-defmethod speechd.repeat ((_driver speechd-driver)))
 
-(cl-defmethod speechd.block ((driver speechd-driver) function))
+(cl-defmethod speechd.block ((_driver speechd-driver) _function))
 
-(cl-defmethod speechd.text ((driver speechd-driver) text cursor))
+(cl-defmethod speechd.text ((_driver speechd-driver) _text _cursor _markers))
 
-(cl-defmethod speechd.icon ((driver speechd-driver) icon))
+(cl-defmethod speechd.icon ((_driver speechd-driver) _icon))
 
-(cl-defmethod speechd.char ((driver speechd-driver) char))
+(cl-defmethod speechd.char ((_driver speechd-driver) _char))
 
-(cl-defmethod speechd.key ((driver speechd-driver) key))
+(cl-defmethod speechd.key ((_driver speechd-driver) _key))
 
-(cl-defmethod speechd.set ((driver speechd-driver) parameter value))
+(cl-defmethod speechd.set ((_driver speechd-driver) _parameter _value))
 
-(cl-defmethod speechd.shutdown ((driver speechd-driver)))
+(cl-defmethod speechd.shutdown ((_driver speechd-driver)))
 
-(defvar speechd.update nil)
+(defvar speechd-update nil)
 
 
 ;;; Interface functions and variables
 
 
-(defstruct speechd-out-update
+(cl-defstruct speechd-out-update
   text
   cursor
   group)
 
 (defmacro speechd-out-with-updated-text (spec &rest body)
-  `(let ((speechd.update ,spec))
+  `(let ((speechd-update ,spec))
      ,@body))
 
 (defun speechd-out-cancel (&optional all)
@@ -166,52 +167,50 @@
   (interactive)
   (speechd-out--loop-drivers-op #'speechd.repeat))
 
-(defun* speechd-out-icon (icon &key (priority speechd-default-sound-priority))
+(cl-defun speechd-out-icon (icon &key (priority speechd-default-sound-priority))
   (let ((icon-name (speechd-out--icon-name icon)))
     (when icon-name
       (speechd-out--loop-drivers (driver)
         (speechd.set driver 'priority priority)
         (speechd.icon driver icon-name)))))
 
-(defun* speechd-out-char (char &key (priority speechd-default-char-priority)
-                               icon)
+(cl-defun speechd-out-char (char &key (priority speechd-default-char-priority) icon)
   (let ((icon-name (speechd-out--icon-name icon)))
     (speechd-out--loop-drivers (driver)
       (speechd.set driver 'priority priority)
-      (lexical-let ((icon-name% icon-name)
-                    (driver% driver)
-                    (char% char))
+      (let ((icon-name% icon-name)
+            (driver% driver)
+            (char% char))
         (speechd.block driver (lambda ()
                                 (when icon-name%
                                   (speechd.icon driver% icon-name%))
                                 (speechd.char driver% char%)))))))
 
-(defun* speechd-out-keys (keys &key (priority speechd-default-key-priority)
-                               text)
+(cl-defun speechd-out-keys (keys &key (priority speechd-default-key-priority) text)
   (speechd-out--loop-drivers (driver)
     (speechd.set driver 'priority priority)
-    (lexical-let ((driver% driver)
-                  (keys% keys)
-                  (text% text))
+    (let ((driver% driver)
+          (keys% keys)
+          (text% text))
       (speechd.block driver (lambda ()
                               (dolist (k keys%)
                                 (speechd.key driver% k))
                               (when text%
-                                (speechd.text driver% text% nil)))))))
+                                (speechd.text driver% text% nil nil)))))))
 
-(defun* speechd-out-text (text &key (priority speechd-default-text-priority)
-                               icon cursor)
+(cl-defun speechd-out-text (text &key (priority speechd-default-text-priority) icon cursor markers)
   (let ((icon-name (speechd-out--icon-name icon)))
     (speechd-out--loop-drivers (driver)
       (speechd.set driver 'priority priority)
-      (lexical-let ((icon-name% icon-name)
-                    (driver% driver)
-                    (text% text)
-                    (cursor% cursor))
+      (let ((icon-name% icon-name)
+            (driver% driver)
+            (text% text)
+            (cursor% cursor)
+            (markers% markers))
         (speechd.block driver (lambda ()
                                 (when icon-name%
                                   (speechd.icon driver% icon-name%))
-                                (speechd.text driver% text% cursor%)))))))
+                                (speechd.text driver% text% cursor% markers%)))))))
 
 (defun speechd-out-set (parameter value)
   (speechd-out--loop-drivers (driver)
@@ -231,7 +230,7 @@
           (completing-read "Enable driver: "
                            (mapcar 'list
                                    (mapcar 'symbol-name
-                                           (set-difference
+                                           (cl-set-difference
                                             (mapcar 'speechd-driver.name
                                                     speechd-out--drivers)
                                             speechd-out-active-drivers)))
