@@ -1,7 +1,7 @@
-;;; speechd-speak.el --- simple speechd-el based Emacs client
+;;; speechd-speak.el --- simple speechd-el based Emacs client  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2010 Brailcom, o.p.s.
-;; Copyright (C) 2012, 2013, 2016, 2019 Milan Zamazal <pdm@zamazal.org>
+;; Copyright (C) 2012-2021 Milan Zamazal <pdm@zamazal.org>
+;; Copyright (C) 2003-2010 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -28,7 +28,8 @@
 ;;; Code:
 
 
-(eval-when-compile (require 'cl))
+(require 'cl-lib)
+(require 'wid-edit)
 
 (require 'speechd)
 (require 'speechd-common)
@@ -175,6 +176,19 @@ and the keys are read after the command is performed."
   :type 'boolean
   :group 'speechd-speak)
 
+(defcustom speechd-speak-use-index-marks nil
+  "If non-nil, move point over text in spoken buffers."
+  :type 'boolean
+  :group 'speechd-speak)
+
+(defcustom speechd-speak-index-mark-regexp ""
+  "Regular expression defining the places where to put index marks to.
+An index mark is put at the end of the regexp at each of the places.
+If the regexp is empty, `sentence-end' is used.  If `sentence-end' is nil,
+a default regexp matching common punctuation is used."
+  :type 'regexp
+  :group 'speechd-speak)
+
 (defcustom speechd-speak-by-properties-on-movement t
   "Method of selection of the piece of text to be spoken on movement.
 Unless a command provides its speechd feedback in a different way, it speaks
@@ -314,7 +328,18 @@ If the current input method is present in the alist, the corresponding language
 is selected unless overridden by another setting."
   :type '(alist :key-type (string :tag "Input method")
                 :value-type (string :tag "Language code"))
-  :group 'speechd)
+  :group 'speechd-speak)
+
+(defcustom speechd-speak-emacs-language "en"
+  "Language to use for texts originating from Emacs.
+Messages, minibuffer prompts, completions often contain English texts,
+so it is desirable to speak them in English.  This variable allows to
+speak them in English, in another language or in the current language.
+The value is an ISO language code string accepted by SSIP.
+If nil, use the current language."
+  :type '(choice (string :tag "Language code")
+                 (const :tag "Current" nil))
+  :group 'speechd-speak)
 
 (defcustom speechd-speak-in-debugger t
   "If nil, speechd-speak functions won't speak in Elisp debuggers.
@@ -339,11 +364,13 @@ This may be useful when debugging speechd-el itself."
   :group 'speechd-speak)
 
 
-;;; Internal constants
+;;; Internal constants and variables
 
 
 (defconst speechd-speak--c-buffer-name "*Completions*")
 
+(defvar speechd-speak--info)
+
 
 ;;; Debugging support
 
@@ -425,7 +452,7 @@ This variable is reset to nil before each command in pre-command-hook.")
                          nil
                        (cons major-mode (buffer-name)))))
     (cond
-     (speechd-speak--client-name-set 
+     (speechd-speak--client-name-set
       speechd-client-name)
      ((and (not speechd-speak--client-name-set)
            (eq speechd-speak-connections speechd-speak--last-connections)
@@ -451,9 +478,9 @@ This variable is reset to nil before each command in pre-command-hook.")
                             result)
                           ;; buffer name
                           (let ((buffer-name (buffer-name)))
-                            (assoc-if (lambda (key)
-                                        (when (stringp key) (string-match key buffer-name)))
-                                      speechd-speak-connections))
+                            (cl-assoc-if (lambda (key)
+                                           (when (stringp key) (string-match key buffer-name)))
+                                         speechd-speak-connections))
                           ;; minibuffer
                           (and (speechd-speak--in-minibuffer-p)
                                (assoc :minibuffer speechd-speak-connections))
@@ -512,7 +539,7 @@ This variable is reset to nil before each command in pre-command-hook.")
 		      ((eq invisibility-spec t)
 		       invisibility)
 		      ((consp invisibility)
-		       (intersection invisibility-spec invisibility))
+		       (cl-intersection invisibility-spec invisibility))
 		      (t
 		       (memq invisibility buffer-invisibility-spec)))))
           (push (substring text pos next-pos) pieces))
@@ -560,12 +587,13 @@ If CHAR is nil, output the character just after current point."
   (speechd-speak--interactive
    (speechd-speak--char (or char (following-char)))))
 
-(defun speechd-speak-read-region (&optional beg end empty-text)
+(defun speechd-speak-read-region (&optional beg end empty-text index-marks)
   "Output region of the current buffer between BEG and END.
 If BEG is nil, current mark is used instead.
 If END is nil, current point is used instead.
 EMPTY-TEXT is a text to output if the region is empty; if nil, empty text icon
-is output."
+is output.
+If INDEX-MARKS is non-nil, insert index marks to the spoken text."
   (interactive "r")
   (speechd-speak--interactive
    (let* ((beg (or beg (mark)))
@@ -580,8 +608,9 @@ is output."
                              :priority speechd-default-text-priority))
       (t
        (let* ((point (point))
-              (cursor (and (>= point beg) (<= point end) (- (point) beg))))
-         (speechd-speak--text text :cursor cursor)))))))
+              (cursor (and (>= point beg) (<= point end) (- (point) beg)))
+              (markers (when index-marks (speechd-speak--markers beg end))))
+         (speechd-speak--text text :cursor cursor :markers markers)))))))
 
 (defun speechd-speak-read-line (&optional rest-only)
   "Output current line.
@@ -592,7 +621,9 @@ to the end of the line."
    (let* ((inhibit-field-text-motion t)
           (in-isearch-p (and isearch-mode (not (called-interactively-p 'interactive))))
           (beg (if (and rest-only (not in-isearch-p)) (point) (line-beginning-position)))
-          (end (line-end-position)))
+          (end (if truncate-lines
+                   (line-end-position)
+                 (save-excursion (end-of-visual-line) (point)))))
      (when speechd-speak-separator-regexp
        (save-match-data
          (save-excursion
@@ -632,13 +663,15 @@ If BUFFER is nil, read current buffer."
    (save-excursion
      (when buffer
        (set-buffer buffer))
-     (speechd-speak-read-region (point-min) (point-max)))))
+     (speechd-speak-read-region (point-min) (point-max) nil
+                                speechd-speak-use-index-marks))))
 
 (defun speechd-speak-read-rest-of-buffer ()
   "Read current buffer from the current point to the end of the buffer."
   (interactive)
   (speechd-speak--interactive
-   (speechd-speak-read-region (point) (point-max))))
+   (speechd-speak-read-region (point) (point-max) nil
+                              speechd-speak-use-index-marks)))
 
 (defun speechd-speak-read-rectangle (beg end)
   "Read text in the region-rectangle."
@@ -701,7 +734,7 @@ This function works only in Emacs 22 or higher."
 (speechd-speak--def-speak-object page)
 (speechd-speak--def-speak-object sexp)
 
-(defstruct speechd-speak--command-info-struct
+(cl-defstruct speechd-speak--command-info-struct
   marker
   modified
   (changes '())
@@ -717,13 +750,13 @@ This function works only in Emacs 22 or higher."
 
 (defmacro speechd-speak--cinfo (slot)
   `(,(speechd-speak--name 'speechd-speak--command-info-struct slot)
-    info))
+    speechd-speak--info))
 
-(defun speechd-speak--command-info-struct-buffer (info)
+(defun speechd-speak--command-info-struct-buffer (_info)
   (let ((marker (speechd-speak--cinfo marker)))
     (and marker (marker-buffer marker))))
 
-(defun speechd-speak--command-info-struct-point (info)
+(defun speechd-speak--command-info-struct-point (_info)
   (let ((marker (speechd-speak--cinfo marker)))
     (and marker (marker-position marker))))
 
@@ -775,21 +808,28 @@ This function works only in Emacs 22 or higher."
   (speechd-speak--set-command-start-info t))
 
 (defmacro speechd-speak--with-command-start-info (&rest body)
-  `(let ((info (speechd-speak--command-start-info)))
-     (when info
+  `(let ((speechd-speak--info (speechd-speak--command-start-info)))
+     (when speechd-speak--info
        ,@body)))
 
 (defmacro speechd-speak--defadvice (function class &rest body)
   (let* ((function* function)
          (functions (if (listp function*) function* (list function*)))
+         (class* class)
+         (where (intern (concat ":" (symbol-name class*))))
+         (aroundp (eq class* 'around))
+         (lambda-list (if aroundp '(orig-fun &rest args) '(&rest args)))
          (aname (if (listp function*) 'speechd-speak 'speechd-speak-user)))
     `(progn
        ,@(mapcar
           (lambda (fname)
-            `(defadvice ,fname (,class ,aname activate preactivate compile)
-               (if (not speechd-speak--started)
-                   ,(when (eq class 'around) 'ad-do-it)
-                 ,@body)))
+            `(define-advice ,fname (,where ,lambda-list ,aname)
+               (cl-flet ,(when aroundp `((call-orig-func () (apply orig-fun args))))
+                 (if (not speechd-speak--started)
+                     ,(if aroundp
+                          `(call-orig-func)
+                        'args)  ;; not nil, to silence warnings about unused `args'
+                   ,@body))))
           functions))))
 
 (defmacro speechd-speak--report (feedback &rest args)
@@ -829,7 +869,7 @@ FUNCTION is invoked interactively."
 			,(if (eq position* 'around)
 			     `(if (called-interactively-p 'interactive)
 				  ,body*
-				ad-do-it)
+				(call-orig-func))
 			   `(when (called-interactively-p 'interactive)
 			      ,body*))))
 		 commands*)
@@ -839,21 +879,19 @@ FUNCTION is invoked interactively."
 (defmacro speechd-speak--command-feedback-region (commands)
   `(speechd-speak--command-feedback ,commands around
      (let ((start (point)))
-       (prog1 ad-do-it
+       (prog1 (call-orig-func)
          (speechd-speak--speak-piece start)))))
 
-(defun* speechd-speak--next-property-change (&optional (point (point))
-                                                       (limit (point-max)))
+(cl-defun speechd-speak--next-property-change (&optional (point (point)) (limit (point-max)))
   (next-char-property-change point limit))
 
-(defun* speechd-speak--previous-property-change (&optional (point (point))
-                                                           (limit (point-min)))
+(cl-defun speechd-speak--previous-property-change (&optional (point (point)) (limit (point-min)))
   ;; Let's be careful about isearch overlays not to cut texts in isearch
   (let* ((i-overlays (and isearch-mode
-                          (intersection (overlays-at (- point 2))
-                                        (append (list isearch-overlay)
-                                                isearch-opened-overlays
-                                                isearch-lazy-highlight-overlays))))
+                          (cl-intersection (overlays-at (- point 2))
+                                           (append (list isearch-overlay)
+                                                   isearch-opened-overlays
+                                                   isearch-lazy-highlight-overlays))))
          (i-overlays-start (1- (apply #'min (1+ point) (mapcar #'overlay-start i-overlays)))))
     (when (>= i-overlays-start limit)
       ;; This may omit other property borders as well.  But in isearch we
@@ -872,16 +910,35 @@ Language must be an RFC 1766 language code, as a string."
   (speechd-set-language language)
   (setq speechd-language language))
 
+(defun speechd-speak--language (&optional special)
+  (or (and special speechd-speak-emacs-language) speechd-language))
+
 (defun speechd-speak--in-comint-p ()
   "Return non-nil if the current buffer is any sort of a comint buffer."
   (and (boundp 'comint-accum-marker)
        comint-accum-marker))
 
+(defvar speechd-speak--marker-counter 0)
+
+(defun speechd-speak--markers (beg end)
+  ;; Return list of (POSITION INDEX MARKER)
+  (let ((regexp speechd-speak-index-mark-regexp)
+        (markers '()))
+    (when (string= regexp "")
+      (setq regexp (or sentence-end "[.,;!?]+")))
+    (save-excursion
+      (goto-char beg)
+      (while (re-search-forward regexp end t)
+        (push (list (cl-incf speechd-speak--marker-counter) (point-marker))
+              markers)))
+    (mapcar #'(lambda (m) (cons (- (marker-position (cl-second m)) beg) m))
+            markers)))
+
 
 ;;; Basic speaking
 
 (speechd-speak--command-feedback (next-line previous-line) after
-  (speechd-speak-read-line))
+  (speechd-speak-read-line (not speechd-speak-whole-line)))
 
 (speechd-speak--command-feedback (forward-word backward-word) after
   (speechd-speak-read-word))
@@ -916,23 +973,23 @@ Language must be an RFC 1766 language code, as a string."
         (let ((speechd-speak--deleted-chars
                (when speechd-speak-deleted-char
                  (speechd-speak--buffer-substring
-                  (max (- (point) (or (ad-get-arg 0) 1))
+                  (max (- (point) (or (cl-first args) 1))
                        (point-min))
                   (point)))))
-          ad-do-it
+          (apply orig-fun args)
           (speechd-speak--store-deleted-chars (if speechd-speak-deleted-char
                                                   speechd-speak--deleted-chars
                                              (format "%c" (preceding-char))))))
-    ad-do-it))
-  
+    (call-orig-func)))
+
 (speechd-speak--defadvice (delete-forward-char delete-char) around
   (let ((speechd-speak--deleted-chars
          (when speechd-speak-deleted-char
            (speechd-speak--buffer-substring
             (point)
-            (min (+ (point) (or (ad-get-arg 0) 1))
+            (min (+ (point) (or (cl-first args) 1))
                  (point-max))))))
-    ad-do-it
+    (call-orig-func)
     (speechd-speak--store-deleted-chars (if speechd-speak-deleted-char
                                             speechd-speak--deleted-chars
                                           (format "%c" (following-char))))))
@@ -959,7 +1016,7 @@ Language must be an RFC 1766 language code, as a string."
     (speechd-speak-read-word)))
 
 (speechd-speak--command-feedback (kill-line) before
-  (speechd-speak-read-line))
+  (speechd-speak-read-line t))
 
 (speechd-speak--command-feedback (kill-sexp) before
   (speechd-speak-read-sexp))
@@ -975,7 +1032,7 @@ Language must be an RFC 1766 language code, as a string."
 
 (speechd-speak--command-feedback (kill-region completion-kill-region) around
   (let ((nlines (count-lines (region-beginning) (region-end))))
-    (prog1 ad-do-it
+    (prog1 (call-orig-func)
       (speechd-speak--maybe-speak*
         (message "Killed region containing %s lines" nlines)))))
 
@@ -992,7 +1049,7 @@ Language must be an RFC 1766 language code, as a string."
   (interactive)
   (speechd-speak--interactive
    (let ((speechd-speak-input-method-languages nil)
-         (speechd-language "en"))
+         (speechd-language (speechd-speak--language t)))
      (speechd-speak--text speechd-speak--last-message))))
 
 (defun speechd-speak--read-message (message)
@@ -1050,8 +1107,8 @@ Language must be an RFC 1766 language code, as a string."
 
 (speechd-speak--defadvice read-from-minibuffer around
   (let ((speechd-speak--minibuffer-inherited-language
-         (and (ad-get-arg 6) speechd-language)))
-    ad-do-it))
+         (and (cl-seventh args) speechd-language)))
+    (call-orig-func)))
 
 (defun speechd-speak--prompt (prompt &optional no-icon)
   (speechd-speak--text prompt :priority 'message
@@ -1062,7 +1119,7 @@ Language must be an RFC 1766 language code, as a string."
    ;; Discard changes, otherwise they would be read *after* the prompt, perhaps
    ;; confusing the user
    (setf (speechd-speak--cinfo changes) '()))
-  (let ((speechd-language "en")
+  (let ((speechd-language (speechd-speak--language t))
         (speechd-speak-input-method-languages nil))
     (speechd-speak--prompt (minibuffer-prompt)))
   (speechd-speak--prompt (minibuffer-contents) t))
@@ -1115,12 +1172,12 @@ This command applies to buffers defined in
 
 (defun speechd-speak--minibuffer-prompt (prompt &rest args)
   (speechd-speak--read-other-changes)
-  (let ((speechd-language "en")
+  (let ((speechd-language (speechd-speak--language t))
         (speechd-speak-input-method-languages nil))
     (apply #'speechd-speak--text prompt args)))
-                          
+
 (speechd-speak--command-feedback minibuffer-message after
-  (speechd-speak--minibuffer-prompt (ad-get-arg 0) :priority 'notification))
+  (speechd-speak--minibuffer-prompt (cl-first args) :priority 'notification))
 
 ;; Some built-in functions, reading a single character answer, prompt in the
 ;; echo area.  They don't invoke minibuffer-setup-hook and may put other
@@ -1139,15 +1196,15 @@ This command applies to buffers defined in
 ;; The following functions don't invoke `minibuffer-setup-hook' and don't put
 ;; the cursor into the echo area.  Sigh.
 (speechd-speak--defadvice read-key-sequence before
-  (let ((prompt (ad-get-arg 0))
+  (let ((prompt (cl-first args))
         (speechd-speak--emulate-minibuffer t))
     (when prompt
       (speechd-speak--minibuffer-prompt prompt :priority 'message))))
 (speechd-speak--defadvice read-event before
-  (let ((prompt (ad-get-arg 0)))
+  (let ((prompt (cl-first args)))
     (when prompt
       (let ((speechd-speak--emulate-minibuffer t)
-            (speechd-language (if (ad-get-arg 1) speechd-language "en")))
+            (speechd-language (speechd-speak--language (not (cl-second args)))))
         (speechd-speak--minibuffer-prompt prompt :priority 'message)))))
 
 
@@ -1166,15 +1223,14 @@ Only single characters are allowed in the keymap.")
   (if speechd-speak-allow-prompt-commands
       (let ((char nil))
         (while (not char)
-          ad-do-it
-          (setq char ad-return-value)
+          (setq char (call-orig-func))
           (let ((command (lookup-key speechd-speak-read-char-keymap
                                      (vector char))))
             (when command
               (setq char nil)
               (call-interactively command))))
         char)
-    ad-do-it))
+    (call-orig-func)))
 
 
 ;;; Commands
@@ -1211,8 +1267,8 @@ Only single characters are allowed in the keymap.")
       (speechd-out-keys (nreverse keys-to-output) :text text-to-output
                         :priority priority))))
 
-(defun speechd-speak--add-command-text (info beg end)
-  (let ((last (first (speechd-speak--cinfo changes)))
+(defun speechd-speak--add-command-text (beg end)
+  (let ((last (cl-first (speechd-speak--cinfo changes)))
         (last-end (speechd-speak--cinfo change-end))
         (text (speechd-speak--buffer-substring beg end t)))
     (setf (speechd-speak--cinfo change-end) end)
@@ -1250,9 +1306,8 @@ Only single characters are allowed in the keymap.")
                                  `(face ,face invisible ,invisible) text)))))
     text))
 
-(defun speechd-speak--minibuffer-update-report (info old new)
+(defun speechd-speak--minibuffer-update-report (old new)
   (speechd-speak--add-command-text
-   info
    (+ (minibuffer-prompt-end)
       (if (and (<= (length old) (length new))
                (string= old (substring new 0 (length old))))
@@ -1260,15 +1315,14 @@ Only single characters are allowed in the keymap.")
         0))
    (point-max)))
 
-(defun speechd-speak--minibuffer-update (beg end len)
+(defun speechd-speak--minibuffer-update ()
   (speechd-speak--with-command-start-info
    (let ((old-content (speechd-speak--cinfo minibuffer-contents))
          (new-content (minibuffer-contents)))
      (unless (or (eq old-content 'unset)
                  (string= old-content new-content))
        (setf (speechd-speak--cinfo minibuffer-contents) new-content)
-       (speechd-speak--minibuffer-update-report
-        info old-content new-content)))))
+       (speechd-speak--minibuffer-update-report old-content new-content)))))
 
 (defun speechd-speak--read-buffer-change (buffer-name text)
   (with-current-buffer (or (get-buffer buffer-name)
@@ -1283,7 +1337,7 @@ Only single characters are allowed in the keymap.")
   (unless speechd-speak--current-change-string
     (setq speechd-speak--current-change-string
           (buffer-substring-no-properties beg end))))
-(defun speechd-speak--after-change-hook (beg end len)
+(defun speechd-speak--after-change-hook (beg end _len)
   (speechd-speak--enforce-speak-mode)
   (speechd-speak--with-command-start-info
     (unless (or (= beg end)
@@ -1314,8 +1368,8 @@ Only single characters are allowed in the keymap.")
         (if (speechd-speak--in-minibuffer-p)
             (progn
               (speechd-speak--read-other-changes)
-              (speechd-speak--minibuffer-update beg end len))
-          (speechd-speak--add-command-text info beg end))))))
+              (speechd-speak--minibuffer-update))
+          (speechd-speak--add-command-text beg end))))))
   (setq speechd-speak--current-change-string nil))
 
 (defconst speechd-speak--dont-cancel-on-commands
@@ -1356,12 +1410,14 @@ Only single characters are allowed in the keymap.")
 
 (defmacro speechd-speak--post-defun (name shy new-state guard &rest body)
   (let* ((name (speechd-speak--name 'speechd-speak--post-read name))
-         (state-condition (case shy
+         (state-condition (cl-case shy
                             ((t) `(eq state nil))
                             (sometimes `(not (eq state t)))
                             (t t))))
     `(defun ,name (state buffer-changed buffer-modified point-moved
                    in-minibuffer other-buffer)
+       ;; Avoid compiler warnings about unused variables
+       (list buffer-changed buffer-modified point-moved in-minibuffer other-buffer)
        (if (and ,state-condition
                 ,guard)
            (let ((speechd-speak--debug-mark (quote ,name)))
@@ -1398,7 +1454,7 @@ Only single characters are allowed in the keymap.")
     (and (stringp deleted-chars)
          (or (null changes)
              (and (= (length changes) 1)
-                  (equal (first changes) deleted-chars)))))
+                  (equal (cl-first changes) deleted-chars)))))
   (when (eq speechd-speak-read-command-keys t)
     ;; Cancel reading the DEL key etc. -- perhaps too daring?
     (speechd-out-cancel))
@@ -1472,8 +1528,8 @@ Only single characters are allowed in the keymap.")
                                       (reverse changes))
                              " ")))
         (when (and self-insert
-                   (> (length (first changes)) 1))
-          (setq text (concat text " " (first changes))))
+                   (> (length (cl-first changes)) 1))
+          (setq text (concat text " " (cl-first changes))))
         (cond
          ((and (eq speechd-speak-buffer-insertions 'whole-buffer)
                (not self-insert))
@@ -1488,7 +1544,7 @@ Only single characters are allowed in the keymap.")
                speechd-speak-echo
                (not (memq 'self-insert-command
                           speechd-speak-ignore-command-keys)))
-      (case speechd-speak-echo
+      (cl-case speechd-speak-echo
         (word
          (let ((point (point)))
            (when (and (> point 1)
@@ -1505,7 +1561,7 @@ Only single characters are allowed in the keymap.")
        (/= (speechd-speak--cinfo completion-buffer-modified)
            (buffer-modified-tick (get-buffer speechd-speak--c-buffer-name))))
   (with-current-buffer speechd-speak--c-buffer-name
-      (let ((speechd-language "en")
+      (let ((speechd-language (speechd-speak--language t))
             (speechd-speak-input-method-languages nil))
         (goto-char (point-min))
         (save-match-data
@@ -1587,7 +1643,7 @@ Only single characters are allowed in the keymap.")
   (and speechd-speak-on-minibuffer-exit
        (/= (minibuffer-depth) speechd-speak--last-minibuffer-depth))
   (speechd-speak-read-line t))
-                           
+
 (defvar speechd-speak--last-minibuffer-depth 0)
 
 (defvar speechd-speak--post-command-speaking-defaults
@@ -1672,7 +1728,7 @@ Only single characters are allowed in the keymap.")
 
 (defun speechd-speak--speak-completion ()
   ;; Taken from `choose-completion'
-  (let (beg end completion (buffer completion-reference-buffer))
+  (let (beg end completion)
     (if (and (not (eobp)) (get-text-property (point) 'mouse-face))
 	(setq end (point) beg (1+ (point))))
     (if (and (not (bobp)) (get-text-property (1- (point)) 'mouse-face))
@@ -1694,7 +1750,7 @@ Only single characters are allowed in the keymap.")
 (speechd-speak--defadvice widget-choose around
   (let ((widget-menu-minibuffer-flag (or speechd-speak-mode
                                          widget-menu-minibuffer-flag)))
-    ad-do-it))
+    (call-orig-func)))
 
 
 ;;; Other functions and packages
@@ -1747,7 +1803,7 @@ When the mode is enabled, all spoken text is spelled."
   (interactive)
   (unless speechd-speak-spell-mode
     (setq speechd-speak-spell-command t)))
-  
+
 
 
 ;;; Informatory commands
@@ -1757,9 +1813,8 @@ When the mode is enabled, all spoken text is spelled."
 
 (defvar speechd-speak--info-updates nil)
 
-(defmacro* speechd-speak--watch (name get-function
-                                 &key on-change info info-string key)
-  `(locally
+(cl-defmacro speechd-speak--watch (name get-function &key on-change info info-string key)
+  `(progn
      (fset (quote ,(speechd-speak--name 'speechd-speak--get name))
            ,get-function)
      ,(when info
@@ -1783,14 +1838,14 @@ When the mode is enabled, all spoken text is spelled."
         `(defun ,(speechd-speak--name 'speechd-speak--update name) (old new)
            (speechd-speak--maybe-speak
             (let ((speechd-default-text-priority 'message)
-                  (speechd-language "en")
+                  (speechd-language (speechd-speak--language t))
                   (speechd-speak-input-method-languages nil))
               (funcall ,on-change old new)))))
      ,(when on-change
         `(add-to-list 'speechd-speak--info-updates (quote ,name)))))
 
 (speechd-speak--watch buffer-name #'buffer-name
-  :on-change #'(lambda (old new)
+  :on-change #'(lambda (_old new)
                  (speechd-speak--text (format "Buffer %s" new))))
 
 (speechd-speak--watch buffer-identification
@@ -1799,17 +1854,17 @@ When the mode is enabled, all spoken text is spelled."
         (let ((ident (format-mode-line mode-line-buffer-identification)))
           (set-text-properties 0 (length ident) nil ident)
           ident)))
-  :on-change #'(lambda (old new)
+  :on-change #'(lambda (_old new)
                  (speechd-speak--text
                   (format "New buffer identification: %s" new))))
 
 (speechd-speak--watch buffer-modified #'buffer-modified-p
-  :on-change #'(lambda (old new)
+  :on-change #'(lambda (_old new)
                  (speechd-speak--text
                   (if new "Buffer modified" "No buffer modification"))))
 
 (speechd-speak--watch buffer-read-only #'(lambda () buffer-read-only)
-  :on-change #'(lambda (old new)
+  :on-change #'(lambda (_old new)
                  (speechd-speak--text
                   (if new "Buffer writable" "Buffer read-only"))))
 
@@ -1839,7 +1894,7 @@ When the mode is enabled, all spoken text is spelled."
         (let ((ident (format-mode-line mode-line-frame-identification)))
           (set-text-properties 0 (length ident) nil ident)
           ident)))
-  :on-change #'(lambda (old new)
+  :on-change #'(lambda (_old new)
                  (speechd-speak--text
                   (format "Frame identification changed to: %s" new))))
 
@@ -1861,7 +1916,7 @@ When the mode is enabled, all spoken text is spelled."
           (let ((line (format-mode-line header-line-format)))
             (if (string= line "") "empty" line))
         "unknown"))
-  :on-change #'(lambda (old new)
+  :on-change #'(lambda (_old new)
                  (speechd-speak--text
                   (format "Header line changed to: %s" new)))
   :info-string "Header line: %s"
@@ -1874,17 +1929,17 @@ When the mode is enabled, all spoken text is spelled."
 
 (speechd-speak--watch minor-modes
   #'(lambda ()
-      (loop for mode-spec in minor-mode-alist
-            for mode = (car mode-spec)
-            when (and (boundp mode) (symbol-value mode))
-            collect (if (memq mode speechd-speak-display-modes)
-                        (cadr mode-spec)
-                      mode)))
+      (cl-loop for mode-spec in minor-mode-alist
+               for mode = (car mode-spec)
+               when (and (boundp mode) (symbol-value mode))
+               collect (if (memq mode speechd-speak-display-modes)
+                           (cadr mode-spec)
+                         mode)))
   :on-change #'(lambda (old new)
                  (cl-flet ((set-difference (x y)
-                          (loop for i in x
-                                unless (memq i y)
-                                collect i)))
+                          (cl-loop for i in x
+                                   unless (memq i y)
+                                   collect i)))
                    (let ((disabled (set-difference old new))
                          (enabled (set-difference new old)))
                      (when enabled
@@ -2051,7 +2106,7 @@ Null prefix argument turns off the mode."
 With no argument, this command toggles the mode.
 Non-null prefix argument turns on the mode.
 Null prefix argument turns off the mode.
-     
+
 When speechd-speak mode is enabled, speech output is provided to Speech
 Dispatcher on many actions.
 
