@@ -1,7 +1,7 @@
-;;; speechd.el --- Library for accessing Speech Dispatcher
+;;; speechd.el --- Library for accessing Speech Dispatcher  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2010 Brailcom, o.p.s.
-;; Copyright (C) 2012, 2013 Milan Zamazal <pdm@zamazal.org>
+;; Copyright (C) 2012-2021 Milan Zamazal <pdm@zamazal.org>
+;; Copyright (C) 2003-2010 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -24,7 +24,7 @@
 
 ;; This library allows you to communicate with Speech Dispatcher.
 ;; Usually, the communication goes like this:
-;; 
+;;
 ;;   (speechd-open)
 ;;   ...
 ;;   (speechd-say-text "Hello, world!")
@@ -39,7 +39,8 @@
 ;;; Code:
 
 
-(eval-when-compile (require 'cl))
+(require 'cl-lib)
+(require 'xml)
 
 (require 'speechd-common)
 
@@ -78,14 +79,6 @@ used for communication with Speech Dispatcher."
   :type 'integer
   :group 'speechd)
 
-(defcustom speechd-spdsend nil
-  "If string, it names the spdsend binary to be used to talk to SSIP.
-If the new version of accept-process-output is available or the value of this
-variable is nil, Emacs talks to an SSIP server (Speech Dispatcher) directly."
-  :type '(choice (const :tag "Don't use spdsend" nil)
-                 (string :tag "spdsend binary"))
-  :group 'speechd)
-
 (defcustom speechd-autospawn t
   "If non-nil, Emacs will attempt to automatically start Speech Dispatcher.
 This means that if speechd-el gets a speech request and the
@@ -118,7 +111,7 @@ launch it."
 
 (defmacro speechd--generate-customization-options (var)
   (let ((value (symbol-value var)))
-    `(quote (choice :value ,(cdr (first value))
+    `(quote (choice :value ,(cdr (cl-first value))
                     ,@(mapcar #'(lambda (o)
                                   `(const :tag ,(car o) ,(cdr o)))
                               value)))))
@@ -131,6 +124,13 @@ launch it."
                                              ("spell" . spell)
                                              ("icon" .  icon)))
 
+(defun speechd--set-variable-and-reopen (name value)
+  (let ((orig-value (when (default-boundp name) (default-value name))))
+    (set-default name value)
+    (when (and (not (equal orig-value value))
+               (fboundp 'speechd-reopen))
+      (speechd-reopen))))
+
 (defcustom speechd-voices '()
   "Alist of voice identifiers and their parameters.
 
@@ -199,10 +199,7 @@ definition of its parameters is optional."
 		  (speechd-priority-tag :value text))
 	    (cons :tag "Output module" (const :format "" output-module)
 		  string))))
-  :set #'(lambda (name value)
-           (set-default name value)
-           (when (fboundp 'speechd-reopen)
-             (speechd-reopen)))           
+  :set #'speechd--set-variable-and-reopen
   :group 'speechd)
 
 (defcustom speechd-connection-voices '()
@@ -217,10 +214,7 @@ For connections that are not specified here, the voice named nil is used.
 You must reopen the connections to apply the changes to this variable."
   :type '(alist :key-type (string :tag "Connection name")
                 :value-type (speechd-voice-tag :tag "Voice"))
-  :set #'(lambda (name value)
-           (set-default name value)
-           (when (fboundp 'speechd-reopen)
-             (speechd-reopen)))           
+  :set #'speechd--set-variable-and-reopen
   :group 'speechd)
 
 (defcustom speechd-cancelable-connections '()
@@ -272,7 +266,9 @@ current voice."
     voice male1
     rate 0
     pitch 0
-    volume 100))
+    volume 100
+    index-marks t
+    ssml-mode nil))
 
 (defconst speechd--coding-system (if (featurep 'xemacs)
 				     'no-conversion-dos ;; No utf yet
@@ -291,6 +287,8 @@ current voice."
     (pitch . "PITCH")
     (volume . "VOLUME")
     (spelling-mode . "SPELLING")
+    (index-marks . "NOTIFICATION INDEX_MARKS")
+    (ssml-mode . "SSML_MODE")
     (output-module . "OUTPUT_MODULE")	; TODO: to be removed sometimes
     ))
 
@@ -298,7 +296,7 @@ current voice."
   '((voices
      "VOICES" identity)
     (synthesis-voices
-     "SYNTHESIS_VOICES" (lambda (line) (first (split-string line))))))
+     "SYNTHESIS_VOICES" (lambda (line) (cl-first (split-string line))))))
 
 (defconst speechd--parameter-value-mappings
   '((message-priority
@@ -317,6 +315,12 @@ current voice."
      (spell . "spell")
      (icon . "icon"))
     (spelling-mode
+     (t . "on")
+     (nil . "off"))
+    (index-marks
+     (t . "on")
+     (nil . "off"))
+    (ssml-mode
      (t . "on")
      (nil . "off"))))
 
@@ -326,7 +330,7 @@ current voice."
 ;;; Internal variables
 
 
-(defstruct speechd--connection
+(cl-defstruct speechd--connection
   name
   host
   port
@@ -340,7 +344,7 @@ current voice."
   (forced-priority nil)
   (last-command nil))
 
-(defstruct speechd--request
+(cl-defstruct speechd--request
   string)
 
 (defvar speechd--connections (make-hash-table :test 'equal)
@@ -348,15 +352,25 @@ current voice."
 
 (defvar speechd--retry-time 1.0)
 
+(defvar speechd-client-name)
+
+(defvar speechd--current-connection)
+
 
 ;;; Utilities
 
 
 (defmacro speechd--iterate-connections (&rest body)
-  `(maphash #'(lambda (_ connection) ,@body) speechd--connections))
+  `(maphash #'(lambda (_ connection)
+                (let ((speechd--current-connection connection))
+                  ,@body))
+            speechd--connections))
 
 (defmacro speechd--iterate-clients (&rest body)
-  `(maphash #'(lambda (speechd-client-name _) ,@body) speechd--connections))
+  `(maphash #'(lambda (name _)
+                (let ((speechd-client-name name))
+                  ,@body))
+            speechd--connections))
 
 (defun speechd-connection-names ()
   "Return the list of all present connection names."
@@ -366,13 +380,13 @@ current voice."
     names))
 
 (defmacro speechd--with-current-connection (&rest body)
-  `(let ((connection (speechd--connection)))
-     (when (and (speechd--connection-failure-p connection)
-                (>= (- (float-time) (speechd--connection-last-try-time connection))
+  `(let ((speechd--current-connection (speechd--connection)))
+     (when (and (speechd--connection-failure-p speechd--current-connection)
+                (>= (- (float-time) (speechd--connection-last-try-time speechd--current-connection))
                     speechd--retry-time))
        (ignore-errors (speechd-reopen t))
-       (setq connection (speechd--connection)))
-     (unless (speechd--connection-failure-p connection)
+       (setq speechd--current-connection (speechd--connection)))
+     (unless (speechd--connection-failure-p speechd--current-connection)
        ,@body)))
 
 (defmacro speechd--with-connection-setting (var value &rest body)
@@ -396,10 +410,10 @@ current voice."
     `(let* ((,$parameters ,parameters)
             (,$orig-parameters ()))
        (unwind-protect
-           (let ((,$cparameters (speechd--connection-parameters connection)))
+           (let ((,$cparameters (speechd--connection-parameters speechd--current-connection)))
              (while ,$parameters
-               (let* ((,$p (first ,$parameters))
-                      (,$v (second ,$parameters))
+               (let* ((,$p (cl-first ,$parameters))
+                      (,$v (cl-second ,$parameters))
                       (,$orig-v (plist-get ,$cparameters ,$p)))
                  (when (and (not (equal ,$v ,$orig-v))
                             (or ,$v
@@ -443,81 +457,84 @@ current voice."
 ;;; Process management functions
 
 
-(defun* speechd--connection (&optional (name speechd-client-name)
-				       (create-if-needed t))
+(cl-defun speechd--connection (&optional (name speechd-client-name) (create-if-needed t))
   (or (gethash name speechd--connections)
       (and create-if-needed
 	   (let ((speechd-client-name name))
 	     (speechd-open)))))
 
-(defvar speechd--advanced-apo (condition-case _
-                                  (progn
-                                    (accept-process-output nil 0 0 1)
-                                    t)
-                                (error)))
+(defun speechd--jump-to-marker (index-mark)
+  (let ((marker (cdr (assoc index-mark speechd--markers))))
+    (when marker
+      (let ((buffer (marker-buffer marker))
+            (position (marker-position marker)))
+        (when buffer
+          (select-window (or (get-buffer-window buffer) (selected-window)))
+          (unless (eq (window-buffer) buffer)
+            (switch-to-buffer buffer))
+          (goto-char position))))))
 
-(defun speechd--call-spdsend (args &optional input)
-  (with-temp-buffer
-    (if input
-        (with-speechd-coding-protection
-          (insert input)
-          (let ((default-process-coding-system
-                  (cons speechd--coding-system speechd--coding-system))
-                (process-coding-system-alist nil))
-            (apply #'call-process-region (point-min) (point-max)
-                   speechd-spdsend t t nil args)))
-      (apply #'call-process speechd-spdsend nil t nil args))
-    (buffer-string)))
+(defun speechd--process-notifications (output)
+  ;; 700-msg_id
+  ;; 700-client_id
+  ;; 700-index_mark
+  ;; 700 END
+  (let ((notifications '())
+        (pos 0))
+    (while (string-match speechd--notification-regexp output pos)
+      (push (list (match-string 1 output) (match-string 2 output) (match-string 3 output))
+            notifications)
+      (setq pos (match-end 0)))
+    ;; There can be additional index marks inserted by Speech Dispatcher,
+    ;; let's skip them and find the first meaningful index mark.
+    (while notifications
+      (cl-destructuring-bind (code separator _message) (car notifications)
+        (setq notifications (cdr notifications))
+        (when (and (equal code speechd--index-mark-code)
+                   (equal separator " "))
+          (when notifications
+            (cl-destructuring-bind (code separator message) (car notifications)
+              (when (and (equal code speechd--index-mark-code)
+                         (equal separator "-"))
+                (speechd--jump-to-marker message)
+                (setq notifications nil))))))))
+  output)
 
-(defun speechd--process-filter (process output)
-  (when speechd--advanced-apo
-    (speechd--with-current-connection
-     (setf (speechd--connection-process-output connection)
-           (concat (speechd--connection-process-output connection) output)))))
-
-(defun speechd--use-spdsend ()
-  (and (not speechd--advanced-apo) speechd-spdsend))
+(defun speechd--process-filter (_process output)
+  (speechd--with-current-connection
+   (setf (speechd--connection-process-output speechd--current-connection)
+         (speechd--process-notifications
+          (concat (speechd--connection-process-output speechd--current-connection) output)))))
 
 (defun speechd--open-connection (method host port socket-name)
-  (if (speechd--use-spdsend)
-      (let* ((answer (speechd--call-spdsend
-                      (list "--open" host (format "%d" port))))
-             (alen (and answer (length answer))))
-	(when (and alen (> alen 1))
-	  (setq answer (substring answer 0 (1- alen))))
-	answer)
-    (let ((process
-           (cond
-            ((eq method 'unix-socket)
-             (make-network-process
-              :name "speechd" :family "local"
-              :remote (or socket-name
-                          (or (getenv "SPEECHD_SOCK")
-                              (expand-file-name
-			       (let ((runtime-dir (getenv "XDG_RUNTIME_DIR")))
-				 (concat (if runtime-dir (concat runtime-dir "/") "~/.")
-                                         "speech-dispatcher/speechd.sock")))))))
-            ((eq method 'inet-socket)
-             (open-network-stream "speechd" nil host port))
-            (t (error "Invalid communication method: `%s'" method)))))
-      (when process
-	(set-process-coding-system process
-				   speechd--coding-system
-				   (if speechd--advanced-apo
-				       speechd--coding-system
-				     'raw-text))
-	(if (fboundp 'set-process-query-on-exit-flag)
-	    (set-process-query-on-exit-flag process nil)
-	  (set-process-query-on-exit-flag process nil))
-	(set-process-filter process #'speechd--process-filter))
-      process)))
+  (let ((process
+         (cond
+          ((eq method 'unix-socket)
+           (make-network-process
+            :name "speechd" :family "local"
+            :remote (or socket-name
+                        (or (getenv "SPEECHD_SOCK")
+                            (expand-file-name
+			     (let ((runtime-dir (getenv "XDG_RUNTIME_DIR")))
+			       (concat (if runtime-dir (concat runtime-dir "/") "~/.")
+                                       "speech-dispatcher/speechd.sock")))))))
+          ((eq method 'inet-socket)
+           (open-network-stream "speechd" nil host port))
+          (t (error "Invalid communication method: `%s'" method)))))
+    (when process
+      (set-process-coding-system process
+				 speechd--coding-system
+				 speechd--coding-system)
+      (if (fboundp 'set-process-query-on-exit-flag)
+	  (set-process-query-on-exit-flag process nil)
+	(set-process-query-on-exit-flag process nil))
+      (set-process-filter process #'speechd--process-filter))
+    process))
 
 (defun speechd--close-connection (connection)
   (let ((process (speechd--connection-process connection)))
     (when process
-      (if (speechd--use-spdsend)
-          (speechd--call-spdsend (list "--close" process))
-        (delete-process (speechd--connection-process connection)))
+      (delete-process (speechd--connection-process connection))
       (setf (speechd--connection-process connection) nil))))
 
 (defun speechd--send-connection (connection command)
@@ -525,22 +542,22 @@ current voice."
     (when process
       (with-speechd-coding-protection
         (condition-case _
-            (if (speechd--use-spdsend)
-                (speechd--call-spdsend (list "--send" process) command)
-              (process-send-string
-               process
-               (if speechd--advanced-apo
-                   command
-                 (encode-coding-string command speechd--coding-system)))
-              (when speechd--advanced-apo
-                (while (not (string-match
-                             speechd--end-regexp
-                             (speechd--connection-process-output connection)))
+            (progn
+              (process-send-string process command)
+              (let ((output (speechd--connection-process-output connection)))
+                (while (not (string-match speechd--end-regexp output))
                   (unless (accept-process-output process speechd-timeout nil 1)
                     (signal 'ssip-connection-error
-                            "Timeout in communication with Speech Dispatcher"))))
-              (prog1 (speechd--connection-process-output connection)
-                (setf (speechd--connection-process-output connection) "")))
+                            "Timeout in communication with Speech Dispatcher"))
+                  (setq output (speechd--connection-process-output connection)))
+                (let ((pos (match-end 0)))
+                  ;; If there are more responses, try to get in sync:
+                  (while (string-match speechd--end-regexp output pos)
+                    (setq output (substring output pos))
+                    (setq pos (- (match-end 0) pos)))
+                  (setf (speechd--connection-process-output connection)
+                        (substring output pos))
+                  (substring output 0 pos))))
           (error
            (speechd--permanent-connection-failure connection)
            (signal 'ssip-connection-error
@@ -552,7 +569,7 @@ current voice."
      "Error on opening Speech Dispatcher connection")
 
 ;;;###autoload
-(defun* speechd-open (&optional method &key host port socket-name quiet force-reopen)
+(cl-defun speechd-open (&optional method &key host port socket-name quiet force-reopen)
   "Open connection to Speech Dispatcher using the given method.
 If the connection corresponding to the current `speechd-client-name' value
 already exists, close it and reopen again, with the same connection parameters.
@@ -605,33 +622,29 @@ Return the opened connection on success, nil otherwise."
                           (t
                            default-parameters)))
              (connection-error nil)
-	     (id (cond
-		  ((or (not connection)
-		       (not (speechd--connection-failure-p connection))
-		       force-reopen)
-                   (condition-case err (speechd--open-connection
-                                        (or method 'unix-socket)
-                                        host port socket-name)
-                     (file-error
-                      (setq connection-error err)
-                      nil)))
-		  ((and connection
-			(not (speechd--connection-failure-p connection)))
-		   (speechd--connection-id connection)))))
-        (when (or id (not quiet))
+	     (process (when (or (not connection)
+		                (not (speechd--connection-failure-p connection))
+		                force-reopen)
+                        (condition-case err (speechd--open-connection
+                                             (or method 'unix-socket)
+                                             host port socket-name)
+                          (file-error
+                           (setq connection-error err)
+                           nil)))))
+        (when (or process (not quiet))
           (setq connection (make-speechd--connection
-                            :name name :host host :port port :process id))
+                            :name name :host host :port port :process process))
           (puthash name connection speechd--connections))
-        (when (and (not id) (not quiet))
+        (when (and (not process) (not quiet))
           (speechd--permanent-connection-failure connection)
           (signal 'ssip-connection-error connection-error))
-	(when id
+	(when process
 	  (speechd--set-connection-name name)
           (setq parameters (append parameters
                                    (list 'language speechd--default-language)))
           (let ((already-set '(client-name)))
             (while parameters
-              (destructuring-bind (parameter value . next) parameters
+              (cl-destructuring-bind (parameter value . next) parameters
                 (unless (memq parameter already-set)
                   (push parameter already-set)
                   (speechd--set-parameter parameter value))
@@ -644,7 +657,7 @@ Return the opened connection on success, nil otherwise."
             (setf (speechd--connection-forced-priority connection) t)))))
     connection))
 
-(defun* speechd-close (&optional (name speechd-client-name))
+(cl-defun speechd-close (&optional (name speechd-client-name))
   "Close speechd connection named NAME."
   (interactive)
   (let ((connection (speechd--connection name nil)))
@@ -665,8 +678,8 @@ If QUIET is non-nil, don't echo success report."
     (maphash #'(lambda (name _)
 		 (let ((speechd-client-name name))
 		   (if (speechd-open nil :quiet t :force-reopen t)
-		       (incf number-of-connections)
-		     (incf number-of-failures))))
+		       (cl-incf number-of-connections)
+		     (cl-incf number-of-failures))))
 	     speechd--connections)
     (unless quiet
       (message (format "%d connections successfully reopened, %d failures"
@@ -677,11 +690,14 @@ If QUIET is non-nil, don't echo success report."
 
 
 (defconst speechd--eol "\n")
-(defconst speechd--end-regexp (format "^[0-9][0-9][0-9] .*%s" speechd--eol))
+(defconst speechd--end-regexp (format "^[0-6][0-9][0-9] .*%s" speechd--eol))
 (defconst speechd--result-regexp
   (format "\\`[0-9][0-9][0-9]-\\(.*\\)%s" speechd--eol))
 (defconst speechd--success-regexp
   (format "^[1-2][0-9][0-9] .*%s" speechd--eol))
+(defconst speechd--notification-regexp
+  (format "^\\(7[0-9][0-9]\\)\\([- ]\\)\\(.*\\)%s" speechd--eol))
+(defconst speechd--index-mark-code "700")
 
 (defun speechd--permanent-connection-failure (connection)
   (speechd--close-connection connection)
@@ -694,26 +710,21 @@ If QUIET is non-nil, don't echo success report."
   (speechd--with-current-connection
    (save-match-data
      (let ((answer (speechd--send-connection
-                    connection (speechd--request-string request))))
+                    speechd--current-connection (speechd--request-string request))))
        (let ((data '())
              success)
          (while (and answer
                      (string-match speechd--result-regexp answer))
            (push (match-string 1 answer) data)
            (setq answer (substring answer (match-end 0))))
-         (if (or speechd--advanced-apo (speechd--use-spdsend))
-             (progn
-               (setq success (and answer
-                                  (string-match speechd--success-regexp
-                                                answer)))
-               (unless success
-                 (setq data nil))
-               (list success
-                     data
-                     (and success (substring answer 0 3))
-                     (and success (substring answer 4))))
-           '(t nil nil nil)))))))
-
+         (setq success (and answer
+                            (string-match speechd--success-regexp answer)))
+         (unless success
+           (setq data nil))
+         (list success
+               data
+               (and success (substring answer 0 3))
+               (and success (substring answer 4))))))))
 
 (defconst speechd--block-commands
   '(("speak")
@@ -723,30 +734,31 @@ If QUIET is non-nil, don't echo success report."
     ("key")
     ("quit")
     ("block" ("end"))
-    ("set" ("self" ("rate") ("pitch") ("volume") ("voice") ("language")))))
+    ("set" ("self" ("rate") ("pitch") ("volume") ("voice") ("language")
+                   ("punctuation") ("cap_let_recogn") ("ssml_mode")))))
 
 (defun speechd--block-command-p (command &optional allowed)
   (unless allowed
     (setq allowed speechd--block-commands))
-  (let* ((match (assoc (downcase (first command)) allowed))
+  (let* ((match (assoc (downcase (cl-first command)) allowed))
          (rest-allowed (cdr match)))
     (and match
          (or (not rest-allowed)
-             (speechd--block-command-p (rest command) rest-allowed)))))
+             (speechd--block-command-p (cl-rest command) rest-allowed)))))
 
-(defun* speechd--send-command (command)
+(cl-defun speechd--send-command (command)
   (unless (listp command)
     (setq command (list command)))
   (speechd--with-current-connection
-    (when (or (not (speechd--connection-in-block connection))
+    (when (or (not (speechd--connection-in-block speechd--current-connection))
               (speechd--block-command-p command))
-      (setf (speechd--connection-last-command connection) command)
+      (setf (speechd--connection-last-command speechd--current-connection) command)
       (speechd--send-request
        (make-speechd--request
         :string (concat (mapconcat #'identity command " ") speechd--eol))))))
 
 (defun speechd--send-text (text)
-  (when (first (or (speechd--send-command "SPEAK") '(t)))
+  (when (cl-first (or (speechd--send-command "SPEAK") '(t)))
     ;; We must be careful here.  There is no answer from SSIP until all data
     ;; including the terminator is sent.  Thus, if we send the data in pieces,
     ;; there may be noticeable delays when waiting for TCP packet
@@ -756,9 +768,8 @@ If QUIET is non-nil, don't echo success report."
       (let ((i 0))
         (while (string-match "[\200-\377]" text i)
           (let ((char (string-to-char (match-string 0 text))))
-            (if (memq (char-charset char)
-                      '(eight-bit-control eight-bit-graphic))
-                (setq text (replace-match (format "\\%o" char) t t text))
+	    (if (char-charset char '(eight-bit))
+                (setq text (replace-match (format "\\%o" (encode-char char 'eight-bit)) t t text))
               (setq i (match-end 0))))))
       (let ((i 0))
         (while (and (> (length text) 0)
@@ -770,10 +781,10 @@ If QUIET is non-nil, don't echo success report."
     ;; We must remove text properties from the string, otherwise Emacs does
     ;; strange things when recoding non-ASCII characters to UTF-8.
     (set-text-properties 0 (length text) nil text)
-    (unless (first (or (speechd--send-request
-                        (make-speechd--request
-                         :string (concat text speechd--eol "." speechd--eol)))
-                       '(t)))
+    (unless (cl-first (or (speechd--send-request
+                           (make-speechd--request
+                            :string (concat text speechd--eol "." speechd--eol)))
+                          '(t)))
       ;; We must reset the connection on failure, to bring it back to the
       ;; command state from the text reading state.
       (speechd-close))))
@@ -783,9 +794,9 @@ If QUIET is non-nil, don't echo success report."
 
 
 (defun speechd--list (parameter)
-  (multiple-value-bind (command processor)
+  (cl-multiple-value-bind (command processor)
       (cdr (assoc parameter speechd--list-parameters))
-    (mapcar processor (second (speechd--send-command (list "LIST" command))))))
+    (mapcar processor (cl-second (speechd--send-command (list "LIST" command))))))
 
 
 ;;; Parameter setting functions
@@ -814,7 +825,7 @@ If QUIET is non-nil, don't echo success report."
     (speechd--send-command (list "SET" (if globally "all" "self") p v))))
 
 (defun speechd--set-connection-parameter (parameter value)
-  (let* ((plist (speechd--connection-parameters connection))
+  (let* ((plist (speechd--connection-parameters speechd--current-connection))
          (orig-value (if (plist-member plist parameter)
                          (plist-get plist parameter)
                        'unknown)))
@@ -822,17 +833,17 @@ If QUIET is non-nil, don't echo success report."
               (and (not (equal orig-value value))
                    (or
                     (not (eq parameter 'message-priority))
-                    (not (speechd--connection-forced-priority connection)))))
+                    (not (speechd--connection-forced-priority speechd--current-connection)))))
       (let ((answer (speechd--call-set-parameter parameter value nil)))
-        (setq connection (speechd--connection))
-        (when (first answer)
-          (setf (speechd--connection-parameters connection)
-                (plist-put (speechd--connection-parameters connection)
+        (setq speechd--current-connection (speechd--connection))
+        (when (cl-first answer)
+          (setf (speechd--connection-parameters speechd--current-connection)
+                (plist-put (speechd--connection-parameters speechd--current-connection)
                            parameter value))))
       ;; Speech Dispatcher bug work-around
       (when (eq parameter 'language)
         (let ((output-module (plist-get
-                              (speechd--connection-parameters connection)
+                              (speechd--connection-parameters speechd--current-connection)
                               'output-module)))
           (when output-module
             (speechd--set-connection-parameter 'output-module
@@ -862,6 +873,11 @@ Language must be an RFC 1766 language code, as a string."
   (interactive (list (read-string "Language: ")))
   (speechd--set-parameter 'language language))
 
+(defun speechd-set-ssml-mode (value)
+  "Set SSML mode value.
+SSML is reset to \"off\" when connection is reopened."
+  (speechd--set-parameter 'ssml-mode value))
+
 (defmacro speechd--generate-set-command (parameter prompt argdesc)
   (let* ((prompt* (concat prompt ": "))
          (argdesc* (eval argdesc))
@@ -930,17 +946,17 @@ If called with a prefix argument, set it for all connections."
               (cons voice
                     (let ((voice-parameters ())
                           (connection-parameters
-                           (speechd--connection-parameters connection)))
+                           (speechd--connection-parameters speechd--current-connection)))
                       (while connection-parameters
-                        (destructuring-bind (p v . next) connection-parameters
+                        (cl-destructuring-bind (p v . next) connection-parameters
                           (unless (memq p '(client-name message-priority
                                             spelling-mode))
                             (push
                              (cons
-                              (case p
+                              (cl-case p
                                 (voice 'name)
                                 (t p))
-                              (case p
+                              (cl-case p
                                 (punctuation-mode
                                  (cdr (assoc
                                        v speechd--punctuation-modes)))
@@ -969,19 +985,20 @@ If called with a prefix argument, set it for all connections."
   "Call FUNCTION inside an SSIP block.
 FUNCTION is called without any arguments."
   (speechd--with-current-connection
-    (let ((%block-connection connection)
-          (nested (and connection (speechd--connection-in-block connection))))
+    (let ((%block-connection speechd--current-connection)
+          (nested (and speechd--current-connection
+                       (speechd--connection-in-block speechd--current-connection))))
       (speechd--with-connection-parameters parameters
-        (let ((speechd-client-name (speechd--connection-name connection)))
+        (let ((speechd-client-name (speechd--connection-name speechd--current-connection)))
           (unless nested
             (speechd--send-command '("BLOCK BEGIN"))
-            (when connection
-              (setf (speechd--connection-in-block connection) t)))
+            (when speechd--current-connection
+              (setf (speechd--connection-in-block speechd--current-connection) t)))
           (unwind-protect (funcall function)
             (unless nested
-              (let ((connection %block-connection))
-                (when connection
-                  (setf (speechd--connection-in-block connection) nil)
+              (let ((speechd--current-connection %block-connection))
+                (when speechd--current-connection
+                  (setf (speechd--connection-in-block speechd--current-connection) nil)
                   (speechd--send-command '("BLOCK END")))))))))))
 
 (defmacro speechd-block* (parameters &rest body)
@@ -997,9 +1014,28 @@ the `speechd--set-parameter' function."
 ;;; Speaking functions
 
 
+(defconst speechd--index-mark-prefix "index-")
+(defvar speechd--markers '())  ; assoc list of (INDEX . MARKER)
+
+(defun speechd--ssml (text beg markers)
+  (let ((end (1- (+ beg (length text)))))
+    (dolist (m markers)
+      (cl-destructuring-bind (pos index _marker) m
+        (when (<= beg pos end)
+          (let ((text-pos (- pos beg)))
+            (put-text-property text-pos (1+ text-pos) 'index index text))))))
+  (setq text (xml-escape-string text))
+  (let ((pos 0))
+    (while (setq pos (next-single-property-change pos 'index text))
+      (let ((mark (format "<mark name=\"%s%s\"/>"
+                          speechd--index-mark-prefix
+                          (get-text-property pos 'index text))))
+        (setq text (concat (substring text 0 pos) mark (substring text pos)))
+        (setq pos (+ pos (length mark) 1)))))
+  (concat "<speak>" text "</speak>"))
+
 ;;;###autoload
-(defun* speechd-say-text (text &key (priority speechd-default-text-priority)
-                               say-if-empty)
+(cl-defun speechd-say-text (text &key (priority speechd-default-text-priority) say-if-empty markers)
   "Speak the given TEXT, represented by a string.
 The key argument `priority' defines the priority of the message and must be one
 of the symbols `important', `message', `text', `notification' or
@@ -1031,14 +1067,23 @@ is empty."
                                              (properties change-point)))))
                         change-point))
                  (substring (substring text beg end)))
+            (when markers
+              (setq speechd--markers
+                    (mapcar #'(lambda (m)
+                                (cons (format "%s%s" speechd--index-mark-prefix
+                                              (cl-second m))
+                                      (cl-third m)))
+                            markers))
+              (setq substring (speechd--ssml substring beg markers)))
             (speechd-block* `(message-priority ,priority
                               language ,(speechd--current-language)
                               spelling-mode ,speechd-spell
+                              ssml-mode ,(not (null markers))
                               ,@properties)
               (speechd--send-text substring))
             (setq beg end)))))))
 
-(defun* speechd-say-sound (name &key (priority speechd-default-sound-priority))
+(cl-defun speechd-say-sound (name &key (priority speechd-default-sound-priority))
   "Ask speechd to play an auditory icon.
 NAME is the name of the icon, any string acceptable by speechd.
 The key argument `priority' defines the priority of the message and must be one
@@ -1047,7 +1092,7 @@ of the symbols `important', `message', `text', `notification' or
   (speechd--set-parameter 'message-priority priority)
   (speechd--send-command (list "SOUND_ICON" name)))
 
-(defun* speechd-say-char (char &key (priority speechd-default-char-priority))
+(cl-defun speechd-say-char (char &key (priority speechd-default-char-priority))
   "Speak the given CHAR, any UTF-8 character.
 The key argument `priority' defines the priority of the message and must be one
 of the symbols `important', `message', `text', `notification' or
@@ -1057,12 +1102,12 @@ of the symbols `important', `message', `text', `notification' or
     (speechd--with-connection-parameters
         `(language ,(speechd--current-language))
       (speechd--send-command
-       (list "CHAR" (format "%s" (case char
+       (list "CHAR" (format "%s" (cl-case char
                                    (?  "space")
                                    (?\n "linefeed")
                                    (t (char-to-string char)))))))))
 
-(defun* speechd-say-key (key &key (priority speechd-default-key-priority))
+(cl-defun speechd-say-key (key &key (priority speechd-default-key-priority))
   "Speak the given KEY, represented by a key event.
 The key argument `priority' defines the priority of the message and must be one
 of the symbols `important', `message', `text', `notification' or
@@ -1077,7 +1122,7 @@ of the symbols `important', `message', `text', `notification' or
                       ((and (>= character 128) (< character 160)) "?")
                       ((= character ? ) "space")
                       ((= character ?_) "underscore")
-                      ((= character ?_) "double-quote")
+                      ((= character ?\") "double-quote")
                       ((= character 127) "backspace")
                       (t (format "%c" character)))
                    (format "%s" character))))
@@ -1097,8 +1142,8 @@ of the symbols `important', `message', `text', `notification' or
   (cond
    ((not all)
     (when (or repeatable
-              (not (equal (first (speechd--connection-last-command
-                                  (speechd--connection)))
+              (not (equal (cl-first (speechd--connection-last-command
+                                     (speechd--connection)))
                           command)))
       (speechd--send-command (list command "self"))))
    ((eq all 'some)
@@ -1108,7 +1153,7 @@ of the symbols `important', `message', `text', `notification' or
                  (member speechd-client-name speechd-cancelable-connections))
          (speechd--control-command command nil repeatable)))))
    ((numberp all)
-    (speechd--iterate-clients 
+    (speechd--iterate-clients
      (speechd--control-command command nil repeatable)))
    (t
     (speechd--send-command (list command "all")))))
@@ -1140,9 +1185,9 @@ If the optional argument ALL is non-nil, pause speaking in all clients."
   (interactive "P")
   (if all
       (speechd--iterate-connections
-        (setf (speechd--connection-paused-p connection) t))
+        (setf (speechd--connection-paused-p speechd--current-connection) t))
     (speechd--with-current-connection
-      (setf (speechd--connection-paused-p connection) t)))
+      (setf (speechd--connection-paused-p speechd--current-connection) t)))
   (speechd--control-command "PAUSE" (not (not all))))
 
 ;;;###autoload
@@ -1152,12 +1197,12 @@ If the optional argument ALL is non-nil, resume speaking messages of all
 clients."
   (interactive "P")
   (speechd--with-current-connection
-    (when (or all (speechd--connection-paused-p connection))
+    (when (or all (speechd--connection-paused-p speechd--current-connection))
       (speechd--control-command "RESUME" (not (not all)))
       (if all
-          (setf (speechd--connection-paused-p connection) nil)
+          (setf (speechd--connection-paused-p speechd--current-connection) nil)
         (speechd--iterate-connections
-          (setf (speechd--connection-paused-p connection) nil))))))
+          (setf (speechd--connection-paused-p speechd--current-connection) nil))))))
 
 ;;;###autoload
 (defun speechd-repeat ()
@@ -1165,8 +1210,8 @@ clients."
   (interactive)
   (if t
       (message "Not yet implemented in speechd")
-    (let ((id (car (second (speechd--send-command
-                            '("HISTORY" "GET" "LAST"))))))
+    (let ((id (car (cl-second (speechd--send-command
+                               '("HISTORY" "GET" "LAST"))))))
       (when id
         (speechd--send-command (list "HISTORY" "SAY" id))))))
 
